package foo;

import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.resource.spi.ActivationSpec;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;
import javax.transaction.xa.XAException;
import javax.resource.spi.BootstrapContext;
import javax.resource.ResourceException;
import javax.resource.spi.work.WorkException;
import javax.resource.spi.work.WorkManager;
import javax.resource.spi.work.ExecutionContext;
import java.util.logging.Logger;

public class FooAdapter implements ResourceAdapter
{
    private Logger logger = Logger.getLogger(this.getClass().toString());
    private FooManager fooManager;
    private BootstrapContext bootstrapContext;

    /**
     * This is called during the activation of a message endpoint.
     *
     * @param endpointFactory
     * @param spec
     */
    public void endpointActivation(MessageEndpointFactory messageEndpointFactory, ActivationSpec spec)
        throws ResourceException
    {
        logger.info("Endpoint activated.");

        fooManager.setMessageEndpointFactory(messageEndpointFactory);

        try
        {
            logger.info("Starting transaction.");
            ExecutionContext ec = startTx();

            bootstrapContext.getWorkManager().startWork(fooManager, 3000, ec, null);
            //bootstrapContext.getWorkManager().startWork(fooManager);

            try {
                bootstrapContext.getXATerminator().commit(ec.getXid(), true);
                logger.info("Committed transaction");
            } catch (XAException e) {
                logger.severe("Failed to start the foo manager."+e);
            }


            logger.info("Foo manager started.");
        }
        catch(WorkException we)
        {
            logger.severe("Failed to start the foo manager."+we);
        }
    }

    protected ExecutionContext startTx() {
            ExecutionContext ec = new ExecutionContext();
            try {
                Xid xid = new XID();
                ec.setXid(xid);
                ec.setTransactionTimeout(60*1000); //5 seconds
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            return ec;
        }


    /**
     * This is called when a message endpoint is deactivated.
     *
     * @param endpointFactory
     * @param spec
     */
    public void endpointDeactivation(MessageEndpointFactory endpointFactory, ActivationSpec spec)
    {
        logger.info("A endpoint is deactivated.");
        fooManager.release();
    }

    /**
     * This method is called by the application server during crash recovery.
     *
     * @param specs
     * @return
     */
    public XAResource[] getXAResources(ActivationSpec[] specs)
    {
        logger.info("Crash recovery initiated.");
        return new XAResource[] {};
    }

    /**
     * This is called when a resource adapter instance is bootstrapped.
     *
     * @param bootstrapContext context
     */
    public void start(BootstrapContext bootstrapContext)
    {
        logger.info("connector bootstrapped");

        this.bootstrapContext = bootstrapContext;

        fooManager = new FooManager();
        fooManager.setWorkManager(bootstrapContext.getWorkManager());
        fooManager.setXATerminator(bootstrapContext.getXATerminator());
    }

    public BootstrapContext getBootstrapContext(){
        return bootstrapContext;
    }

    /**
     * This is called when a resource adapter instance is undeployed or during application server shutdown.
     */
    public void stop()
    {
        logger.info("The connector is shutting down.");
        fooManager.release();
    }
}
