package foo;

import java.util.ArrayList;
import javax.resource.spi.endpoint.MessageEndpoint;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.resource.spi.work.WorkManager;
import javax.resource.spi.work.Work;
import javax.resource.spi.work.ExecutionContext;
import javax.resource.spi.XATerminator;
import javax.transaction.xa.Xid;
import java.util.logging.Logger;

public class FooManager implements Work
{
    private Logger logger = Logger.getLogger(this.getClass().toString());
    private WorkManager workManager;
    private MessageEndpointFactory messageEndpointFactory;
    private volatile boolean run = true;
    private XATerminator xa;

    public void setWorkManager(WorkManager workManager)
    {
        this.workManager = workManager;
    }

    public void setXATerminator(XATerminator xa){
        this.xa = xa;
    }

    public void setMessageEndpointFactory(MessageEndpointFactory messageEndpointFactory)
    {
        this.messageEndpointFactory = messageEndpointFactory;
    }
    protected ExecutionContext startTx() {
            ExecutionContext ec = new ExecutionContext();
            try {
                Xid xid = new XID();
                ec.setXid(xid);
                ec.setTransactionTimeout(30*1000); //5 seconds
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            return ec;
        }
    
    public void run()
    {
        logger.info("manager run");
        FooEndpoint foo1 = null;
        FooEndpoint foo2 = null;
        SomeWork work = null;
        MessageEndpoint endpoint1 = null;
        MessageEndpoint endpoint2 = null;
        try
        {

            /* Create new endpoint for own use, will end up being a new
               MessageEndpointInvocationHandler instance but use the same
               underlying FooEndpointImpl as the next one created */
            endpoint1 = messageEndpointFactory.createEndpoint(null);
            foo1 = (FooEndpoint)endpoint1;
            foo1.init();
            logger.info("foo1 endpoint: " + endpoint1);
            
            work = new SomeWork();
            /* Create foo2 for subwork */
            endpoint2 = messageEndpointFactory.createEndpoint(null);
            foo2 = (FooEndpoint)endpoint2;

            //ExecutionContext ec = startTx();
            foo2.init();
            logger.info("foo2 endpoint: " + endpoint2);
            
            work.setEndpoint(foo2);
            workManager.startWork(work,5000,null, null);//startWork(work);

            /* These two calls will end up on the same underlying FooEndpointImpl */
            logger.info("calling " + foo1);
            foo1.doNothing1("MANAGER");
            logger.info("calling " + foo2);
            foo2.doNothing2("MANAGER");

            while (run)
            {
                Thread.sleep(1000);
            }

            //xa.commit(ec.getXid(), true);
        }
        catch (Exception e)
        {
            logger.severe("manager failed"+e);
        }
        finally
        {
            if (endpoint1 != null){
                logger.info("releasing endpoint1");
                endpoint1.release();
            }
            if (endpoint2 != null){
                endpoint2.release();
                logger.info("releasing endpoint2");
            }
        }
        logger.info("manager done");
    }

    public void release()
    {
        run = false;
    }
}
